/*
 * Copyright (c) 2023-2024 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Fri Nov 17 2023
 */
#ifndef H_CAMERA_GENOM3
#define H_CAMERA_GENOM3

#ifdef __cplusplus
# include <time.h>

# include <vector>
# include <functional>

struct camera_slot;
struct camera_slot_image {
  or_image_frame *image{nullptr};
  struct timespec ts{0, 0};
};

struct camera_job {
  typedef std::vector<camera_slot *> slots;
  enum state { idle, update, cancel };

  camera_job() {};
  camera_job(const camera_job &) = delete;
  void operator=(const camera_job &) = delete;
  virtual ~camera_job();
  virtual void destroy() = 0;

  genom_event status(camera_einval_detail &detail) const;

  camera_slot *src(const std::string &name);
  camera_slot *sink(const std::string &name);
  void reset();

  bool schedule(const camera_pipeline *pipeline);

  bool all_input_recent() const;
  virtual bool runnable() const = 0;

  size_t nin() const;
  const camera_slot_image *in(size_t index) const;
  size_t nout() const;
  const camera_slot_image *out(size_t index) const;

 protected:
  std::string errs{};

 private:
  friend camera_pipeline;

  bool lock();
  void unlock(bool update);

  virtual state operator()() = 0;

  struct timespec ts{0, 0};
  struct { slots in, out; } current, locked;
  camera_job **ref{nullptr};
};

#endif /* __cplusplus */

#endif /* H_CAMERA_GENOM3 */
